package net.knarcraft.knarlib.particle;

import net.knarcraft.knarlib.util.ParticleHelper;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.UUID;

/**
 * A task for spawning trails behind players
 */
@SuppressWarnings("unused")
public final class ParticleTrailSpawner implements Runnable {

    private final Set<UUID> playersWithTrails = new HashSet<>();
    private final Map<UUID, Particle> playerParticles = new HashMap<>();
    private final Random random = new Random();
    private final Particle particle;
    private final List<Particle> randomTrailTypes;

    /**
     * Instantiates a new particle trail spawner
     *
     * @param particle         <p>The type of particle used for the trail</p>
     * @param randomTrailTypes <p>The types of particles to use for random trails</p>
     */
    public ParticleTrailSpawner(@NotNull Particle particle, @NotNull List<Particle> randomTrailTypes) {
        this.particle = particle;
        this.randomTrailTypes = randomTrailTypes;
    }

    @Override
    public void run() {
        Set<UUID> offlinePlayers = new HashSet<>();
        for (UUID playerId : playersWithTrails) {
            // Clear offline players from the list
            Player player = Bukkit.getPlayer(playerId);
            if (player == null) {
                offlinePlayers.add(playerId);
                continue;
            }

            // Get the world and location to spawn the particle at
            Location playerLocation = player.getLocation();
            World playerWorld = playerLocation.getWorld();
            if (playerWorld == null) {
                continue;
            }

            // Decide on which type of particle to spawn
            Particle spawnParticle = playerParticles.get(playerId);
            if (spawnParticle == null) {
                spawnParticle = this.particle;
            }

            // Spawn a trail particle
            ParticleConfig particleConfig = new ParticleConfigBuilder(spawnParticle).build();
            ParticleHelper.spawnParticle(playerWorld, playerLocation, particleConfig, 0);
        }
        playersWithTrails.removeAll(offlinePlayers);

    }

    /**
     * Removes the trail behind the player with the given id
     *
     * @param playerId <p>The id of the player to remove the trail for</p>
     */
    public void removeTrail(@NotNull UUID playerId) {
        this.playersWithTrails.remove(playerId);
        this.playerParticles.remove(playerId);
    }

    /**
     * Adds a random trail behind the player with the given id
     *
     * @param playerId <p>The id of the player to add the trail to</p>
     */
    public void startRandomTrail(@NotNull UUID playerId) {
        this.playerParticles.put(playerId, randomParticle());
        this.playersWithTrails.add(playerId);
    }

    /**
     * Adds a trail behind the player with the given id
     *
     * @param playerId <p>The id of the player to add the trail to</p>
     * @param particle <p>The particle to spawn, or null for the particle specified in the constructor</p>
     */
    public void startTrail(@NotNull UUID playerId, @Nullable Particle particle) {
        this.playerParticles.put(playerId, particle);
        this.playersWithTrails.add(playerId);
    }

    /**
     * Gets a random particle from random the specified trail types
     *
     * @return <p>A random particle, or EGG_CRACK if no random trail types have been specified</p>
     */
    private @NotNull Particle randomParticle() {
        if (this.randomTrailTypes.isEmpty()) {
            return Particle.EGG_CRACK;
        }

        Particle spawnParticle = null;
        while (spawnParticle == null || spawnParticle.getDataType() != Void.class) {
            spawnParticle = randomTrailTypes.get(random.nextInt(randomTrailTypes.size()));
        }
        return spawnParticle;
    }

}
